@@ -1,12 +1,11 @@
-use bellperson::bls::Fr;
-use ff::Field;
+use blstrs::Scalar as Fr;
 use filecoin_hashers::Domain;
 
 pub fn encode<T: Domain>(key: T, value: T) -> T {
     let mut result: Fr = value.into();
     let key: Fr = key.into();
 
-    result.add_assign(&key);
+    result += key;
     result.into()
 }
 
@@ -14,6 +13,6 @@ pub fn decode<T: Domain>(key: T, value: T) -> T {
     let mut result: Fr = value.into();
     let key: Fr = key.into();
 
-    result.sub_assign(&key);
+    result -= key;
     result.into()
 }
