@@ -112,10 +112,12 @@ where
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct PublicInputs<T: Domain, S: Domain> {
+    #[serde(bound = "")]
     pub replica_id: T,
     pub seed: [u8; 32],
+    #[serde(bound = "")]
     pub tau: Option<Tau<T, S>>,
     /// Partition index
     pub k: Option<usize>,
@@ -327,9 +329,11 @@ pub type TransformedLayers<Tree, G> = (
 );
 
 /// Tau for a single parition.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub struct Tau<D: Domain, E: Domain> {
+    #[serde(bound = "")]
     pub comm_d: E,
+    #[serde(bound = "")]
     pub comm_r: D,
 }
 
@@ -407,7 +411,7 @@ impl<Tree: MerkleTreeTrait, G: Hasher> TemporaryAux<Tree, G> {
             let tree_c_store = DiskStore::<<Tree::Hasher as Hasher>::Domain>::new_from_disk(
                 tree_c_size,
                 Tree::Arity::to_usize(),
-                &config,
+                config,
             )
             .context("tree_c")?;
             // Note: from_data_store requires the base tree leaf count
@@ -669,7 +673,7 @@ impl<Tree: MerkleTreeTrait> Labels<Tree> {
                 let store = DiskStore::new_from_disk(
                     label.size.expect("label size failure"),
                     Tree::Arity::to_usize(),
-                    &label,
+                    label,
                 )?;
                 store.read_at(node as usize)
             })
