@@ -1,3 +1,4 @@
+use std::any::TypeId;
 use std::fs;
 use std::marker::PhantomData;
 use std::panic::panic_any;
@@ -7,10 +8,10 @@ use std::sync::Mutex;
 use anyhow::Context;
 use bincode::deserialize;
 use fdlimit::raise_fd_limit;
-use filecoin_hashers::{Domain, HashFunction, Hasher, PoseidonArity};
+use filecoin_hashers::{poseidon::PoseidonHasher, Domain, HashFunction, Hasher, PoseidonArity};
 use generic_array::typenum::{Unsigned, U0, U11, U2, U8};
 use lazy_static::lazy_static;
-use log::{error, info, trace};
+use log::{error, info, trace, warn};
 use merkletree::{
     merkle::{get_merkle_tree_len, is_merkle_tree_size_valid},
     store::{Store, StoreConfig},
@@ -242,7 +243,7 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
                             {
                                 let labeled_node = rcp.c_x.get_node_at_layer(layer)?;
                                 assert!(
-                                    proof.verify(&pub_inputs.replica_id, &labeled_node),
+                                    proof.verify(&pub_inputs.replica_id, labeled_node),
                                     "Invalid encoding proof generated at layer {}",
                                     layer,
                                 );
@@ -428,7 +429,9 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
         ColumnArity: 'static + PoseidonArity,
         TreeArity: PoseidonArity,
     {
-        if SETTINGS.use_gpu_column_builder {
+        if SETTINGS.use_gpu_column_builder
+            && TypeId::of::<Tree::Hasher>() == TypeId::of::<PoseidonHasher>()
+        {
             Self::generate_tree_c_gpu::<ColumnArity, TreeArity>(
                 layers,
                 nodes_count,
@@ -591,14 +594,20 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
                 });
                 s.execute(move || {
                     let _gpu_lock = GPU_LOCK.lock().expect("failed to get gpu lock");
-                    let tree_batcher = Some(
-                        Batcher::pick_gpu(max_gpu_tree_batch_size)
-                            .expect("failed to create tree gpu batcher"),
-                    );
-                    let column_batcher = Some(
-                        Batcher::pick_gpu(max_gpu_column_batch_size)
-                            .expect("failed to create col gpu batcher"),
-                    );
+                    let tree_batcher = match Batcher::pick_gpu(max_gpu_tree_batch_size) {
+                        Ok(b) => Some(b),
+                        Err(err) => {
+                            warn!("no GPU found, falling back to CPU tree builder: {}", err);
+                            None
+                        }
+                    };
+                    let column_batcher = match Batcher::pick_gpu(max_gpu_column_batch_size) {
+                        Ok(b) => Some(b),
+                        Err(err) => {
+                            warn!("no GPU found, falling back to CPU tree builder: {}", err);
+                            None
+                        }
+                    };
                     let mut column_tree_builder = ColumnTreeBuilder::<ColumnArity, TreeArity>::new(
                         column_batcher,
                         tree_batcher,
@@ -686,7 +695,7 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
                                 let mut buf = Vec::with_capacity(batch_size * NODE_SIZE);
 
                                 for fr in fr_elements {
-                                    buf.extend(fr_into_bytes(&fr));
+                                    buf.extend(fr_into_bytes(fr));
                                 }
                                 store
                                     .write()
@@ -806,7 +815,10 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
     where
         TreeArity: PoseidonArity,
     {
-        if SETTINGS.use_gpu_tree_builder {
+        // The GPU tree builder only support Poseidon hashes.
+        if SETTINGS.use_gpu_tree_builder
+            && TypeId::of::<Tree::Hasher>() == TypeId::of::<PoseidonHasher>()
+        {
             Self::generate_tree_r_last_gpu::<TreeArity>(
                 data,
                 nodes_count,
@@ -973,10 +985,13 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
             });
             s.execute(move || {
                 let _gpu_lock = GPU_LOCK.lock().expect("failed to get gpu lock");
-                let batcher = Some(
-                    Batcher::pick_gpu(max_gpu_tree_batch_size)
-                        .expect("failed to create gpu batcher"),
-                );
+                let batcher = match Batcher::pick_gpu(max_gpu_tree_batch_size) {
+                    Ok(b) => Some(b),
+                    Err(err) => {
+                        warn!("no GPU found, falling back to CPU tree builder: {}", err);
+                        None
+                    }
+                };
                 let mut tree_builder = TreeBuilder::<Tree::Arity>::new(
                     batcher,
                     nodes_count,
@@ -1057,7 +1072,7 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
 
         create_lc_tree::<LCTree<Tree::Hasher, Tree::Arity, Tree::SubTreeArity, Tree::TopTreeArity>>(
             tree_r_last_config.size.expect("config size failure"),
-            &configs,
+            configs,
             &replica_config,
         )
     }
@@ -1437,9 +1452,13 @@ impl<'a, Tree: 'static + MerkleTreeTrait, G: 'static + Hasher> StackedDrg<'a, Tr
             let max_gpu_tree_batch_size = SETTINGS.max_gpu_tree_batch_size as usize;
 
             let _gpu_lock = GPU_LOCK.lock().expect("failed to get gpu lock");
-            let batcher = Some(
-                Batcher::pick_gpu(max_gpu_tree_batch_size).expect("failed to create gpu batcher"),
-            );
+            let batcher = match Batcher::pick_gpu(max_gpu_tree_batch_size) {
+                Ok(b) => Some(b),
+                Err(err) => {
+                    warn!("no GPU found, falling back to CPU tree builder: {}", err);
+                    None
+                }
+            };
             let mut tree_builder = TreeBuilder::<Tree::Arity>::new(
                 batcher,
                 nodes_count,
